@@ -46,6 +46,7 @@ public:
   int trim = 0;                      // output only reflections -N<=h,k,l<=N
   int free_flag_value = -1;          // -1 = auto: 0 or (if we have >50% of 0's) 1
   std::string staraniso_version;     // for _software.version in "special_marker"
+  std::string gemmi_run_from;        // added to gemmi as _software.description
 
   static const char** default_spec(bool for_merged) {
     static const char* merged[] = {
@@ -92,6 +93,8 @@ public:
                  SMat33<double>* staraniso_b, std::ostream& os);
   void write_cif_from_xds(const XdsAscii& xds, std::ostream& os);
 
+  void write_staraniso_b_in_mmcif(const SMat33<double>& b, char* buf, std::ostream& os);
+
 private:
   // describes which MTZ column is to be translated to what mmCIF column
   struct Trans {
@@ -115,9 +118,7 @@ private:
   std::vector<SweepData> sweeps;
   std::unordered_map<int, int> sweep_indices;
 
-  std::vector<Trans> recipe;
-
-  const Trans* get_status_translation() const {
+  static const Trans* get_status_translation(const std::vector<Trans>& recipe) {
     for (const Trans& t: recipe)
       if (t.is_status)
         return &t;
@@ -225,16 +226,16 @@ private:
     bool discard_next_line = false;
   };
 
-  void prepare_recipe(const Mtz& mtz) {
+  void prepare_recipe(const Mtz& mtz, std::vector<Trans>& recipe) const {
     recipe.clear();
     SpecParserState state;
     if (!spec_lines.empty()) {
       for (const std::string& line : spec_lines)
-        parse_spec_line(line.c_str(), mtz, state);
+        parse_spec_line(recipe, line.c_str(), mtz, state);
     } else {
       const char** lines = default_spec(/*for_merged=*/mtz.batches.empty());
       for (; *lines != nullptr; ++lines)
-        parse_spec_line(*lines, mtz, state);
+        parse_spec_line(recipe, *lines, mtz, state);
     }
     if (recipe.empty())
       fail("empty translation recipe");
@@ -254,7 +255,8 @@ private:
   }
 
   // adds results to recipe
-  void parse_spec_line(const char* line, const Mtz& mtz, SpecParserState& state) {
+  void parse_spec_line(std::vector<Trans>& recipe, const char* line,
+                       const Mtz& mtz, SpecParserState& state) const {
     Trans tr;
     const char* p = line;
     if (*p == '&') {
@@ -345,20 +347,28 @@ private:
     if (!write_special_marker_for_pdb)
       return;
     os << "### IF YOU MODIFY THIS FILE, REMOVE THIS SIGNATURE: ###\n";
+    std::string desc;
+    if (!gemmi_run_from.empty())
+      desc = " 'run from " + gemmi_run_from + "'";
     if (!merged || staraniso_version.empty()) {
       os << "_software.pdbx_ordinal 1\n"
             "_software.classification 'data extraction'\n"
             "_software.name gemmi\n"
             "_software.version " GEMMI_VERSION "\n";
+      if (!desc.empty())
+        os << "_software.description" << desc << '\n';
     } else {
       os << "loop_\n"
             "_software.pdbx_ordinal\n"
             "_software.classification\n"
             "_software.name\n"
-            "_software.version\n"
-            "1 'data extraction' gemmi " GEMMI_VERSION "\n";
+            "_software.version\n";
+      if (!desc.empty())
+        os << "_software.description\n";
+      os << "1 'data extraction' gemmi " GEMMI_VERSION << desc << '\n';
       // STARANISO here tells that intensities were scaled anisotropically.
-      os << "2 'data scaling' STARANISO '" << staraniso_version << "'\n";
+      os << "2 'data scaling' STARANISO '" << staraniso_version
+         << (!desc.empty() ? "' .\n" : "'\n");
     }
     os << "_pdbx_audit_conform.dict_name mmcif_pdbx.dic\n"
           "_pdbx_audit_conform.dict_version 5.339\n"
@@ -371,9 +381,37 @@ private:
                                const SpaceGroup* sg,
                                char* buf, std::ostream& os) const;
 
-  void write_main_loop(const Mtz& mtz, char* buf, std::ostream& os);
+  void write_main_loop(const Mtz& mtz, const std::vector<Trans>& recipe,
+                       char* buf, std::ostream& os);
 };
 
+// remove '_dataset_name' that can appended to column names in ccp4i
+inline void remove_appendix_from_column_names(Mtz& mtz, std::ostream& out) {
+  std::string appendix;
+  for (char type : {'J', 'F'}) {  // intensity, amplitude
+    auto cols = mtz.columns_with_type(type);
+    // Remove the appendix only in clear cases. If mtz has multiple
+    // intensity columns they could have IMEAN_this and IMEAN.
+    if (cols.size() == 1) {
+      auto pos = cols[0]->label.find('_');
+      if (pos == std::string::npos)
+        return;
+      appendix = cols[0]->label.substr(pos);
+      break;
+    }
+  }
+  out << "Ignoring '" << appendix << "' appended to column names.\n";
+  for (Mtz::Column& col : mtz.columns) {
+    size_t from_end  = appendix.size();
+    // the appendix can before (+)/(-), i.e. I_appendix(+)
+    if (!col.label.empty() && col.label.back() == ')')
+      from_end += 3;
+    if (from_end < col.label.size() &&
+        col.label.compare(col.label.size() - from_end, appendix.size(), appendix) == 0)
+      col.label.erase(col.label.size() - from_end, appendix.size());
+  }
+}
+
 inline bool validate_merged_mtz_deposition_columns(const Mtz& mtz, std::ostream& out) {
   bool ok = true;
   if (!mtz.rfree_column()) {
@@ -580,15 +618,22 @@ inline void reorder_staraniso_eigensystem(Mat33& vectors, double (&values)[3]) {
   std::memcpy(values, tmp, sizeof(tmp));
 }
 
-inline void write_staraniso_b_in_mmcif(const SMat33<double>& b,
-                                       char* buf, std::ostream& os) {
+inline void MtzToCif::write_staraniso_b_in_mmcif(const SMat33<double>& b,
+                                                 char* buf, std::ostream& os) {
   double eigenvalues[3];
   Mat33 eigenvectors = eigen_decomposition(b, eigenvalues);
   reorder_staraniso_eigensystem(eigenvectors, eigenvalues);
+  // three mandatory items in _reflns
+  os << "\n_reflns.entry_id " << entry_id
+     << "\n_reflns.pdbx_ordinal 1"
+        "\n_reflns.pdbx_diffrn_id 1";
   const char* prefix = "\n_reflns.pdbx_aniso_B_tensor_eigen";
   for (int i = 0; i < 3; ++i) {
-    double v = std::fabs(eigenvalues[i]) > 1e-4 ? eigenvalues[i] : 0;
-    WRITE("%svalue_%d %.5g", prefix, i+1, v);
+    // According to the mmCIF spec eigenvalues must be positive.
+    // But we don't have the original eigenvalues, we have values after
+    // subtracting the smallest eigenvalue. So at least one of them is 0.
+    // As a workaround, add an arbitrary number to all eigenvalues.
+    WRITE("%svalue_%d %.5g", prefix, i+1, eigenvalues[i] + 1.0);
     for (int j = 0; j < 3; ++j)
       WRITE("%svector_%d_ortho[%d] %.5g", prefix, i+1, j+1, eigenvectors[j][i]);
   }
@@ -622,6 +667,10 @@ inline void MtzToCif::write_cif(const Mtz& mtz, const Mtz* mtz2,
 
   write_special_marker_if_requested(os, merged);
 
+  std::vector<Trans> recipe;
+  if (merged)
+    prepare_recipe(*merged, recipe);
+
   if (unmerged) {
     bool ok = gather_sweep_data(*unmerged);
     std::set<const Mtz::Dataset*> used_datasets;
@@ -706,12 +755,10 @@ inline void MtzToCif::write_cif(const Mtz& mtz, const Mtz* mtz2,
       }
       os << '\n';
     }
-  } else {
+  } else {  // not unmerged, i.e. only merged data
     double w = std::isnan(wavelength) ? get_wavelength(mtz, recipe) : wavelength;
     if (w > 0.)
-      os << "_diffrn_radiation.diffrn_id 1\n"
-         << "_diffrn_radiation.wavelength_id 1\n"
-         << "_diffrn_radiation_wavelength.id 1\n"
+      os << "_diffrn_radiation_wavelength.id 1\n"
          << "_diffrn_radiation_wavelength.wavelength " << to_str(w) << "\n\n";
   }
 
@@ -727,13 +774,15 @@ inline void MtzToCif::write_cif(const Mtz& mtz, const Mtz* mtz2,
     write_staraniso_b_in_mmcif(*staraniso_b, buf, os);
 
   if (merged)
-    write_main_loop(*merged, buf, os);
-  if (unmerged)
-    write_main_loop(*unmerged, buf, os);
+    write_main_loop(*merged, recipe, buf, os);
+  if (unmerged) {
+    prepare_recipe(*unmerged, recipe);
+    write_main_loop(*unmerged, recipe, buf, os);
+  }
 }
 
-inline void MtzToCif::write_main_loop(const Mtz& mtz, char* buf, std::ostream& os) {
-  prepare_recipe(mtz);
+inline void MtzToCif::write_main_loop(const Mtz& mtz, const std::vector<Trans>& recipe,
+                                      char* buf, std::ostream& os) {
   // prepare indices
   std::vector<int> value_indices;  // used for --skip_empty
   std::vector<int> sigma_indices;  // used for status 'x'
@@ -776,7 +825,7 @@ inline void MtzToCif::write_main_loop(const Mtz& mtz, char* buf, std::ostream& o
   if (free_flag_value < 0) {
     // CCP4 uses flags 0,...N-1 (usually N=20), with default free set 0
     // PHENIX uses 0/1 flags with free set 1
-    if (const Trans* tr_status = get_status_translation()) {
+    if (const Trans* tr_status = get_status_translation(recipe)) {
       int count = 0;
       for (float val : mtz.columns[tr_status->col_idx])
         if (val == 0.f)
